@@ -60,9 +60,9 @@ setClass("CoImp",
 setMethod(
     f="plot",
     signature(x = "CoImp",y = "missing"),
-    definition=function(x, y, plot.legend = TRUE, args.legend = list(y = 110, cex = 0.8),...){
+    definition=function(x, y, plot.legend = TRUE, args.legend = list(y = 110, cex = 0.8), plot.name = NULL, ...){
         par(mfrow=c(1,1));
-        bar.plot(tab = x@Perc.miss, legend.plot = plot.legend, args.legend = args.legend)
+        bar.plot(tab = x@Perc.miss, legend.plot = plot.legend, args.legend = args.legend, ...)
         X <- x@Imputed.data.matrix
         smoothing <- x@Smooth.param
         n.marg <- ncol(X)
@@ -80,25 +80,34 @@ setMethod(
             f.x[[j]] <- function(x){predict(fit0[[j]], newdata=x)/as.numeric(knorm[[1]])}
         }
         #
-        if(n.marg<=6){
+        if(n.marg<=3){
             dev.new();
-            par(mfrow=c(2,ceiling(n.marg/2)))
+            par(mfrow=c(1,ceiling(n.marg)))
         }else{
-            dev.new();
-            par(mfrow=c(3,ceiling(n.marg/3)))
+            if(n.marg >3 && n.marg<=6){
+                dev.new();
+                par(mfrow=c(2,ceiling(n.marg/2)))
+            }else{
+                dev.new();
+                par(mfrow=c(3,ceiling(n.marg/3)))
+            }
         }
         par(mai=c(0.5,0.5,0.3,0.5))
         for(i in 1:n.marg){
             j       <- i
-            minimo  <- min(X[complete.cases(X[,i]),i])-1
-            massimo <- max(X[complete.cases(X[,i]),i])+1
+            minimo  <- min(X[complete.cases(X[,i]),i])
+            massimo <- max(X[complete.cases(X[,i]),i])
             opt1     <- f.x[[i]](optimize(f.x[[i]], c(minimo,massimo), maximum=TRUE)$maximum)
             his  <- hist(X[,i], plot=FALSE)
             opt2 <- max(his$density)
             opt  <- max(opt1,opt2)
-            plot(his, freq=FALSE, xlim=c(minimo, massimo), ylim=c(0,opt), main="", xlab="", ylab="")
-            plot(f.x[[i]], lwd=2, xlim=c(minimo, massimo), ylim=c(0,opt), col="blue", add=TRUE)
-            title(main = paste("Variable X", i,sep=""))
+            plot(his, freq=FALSE, ylim=c(0,opt), main="", xlab="", ylab="")
+            plot(f.x[[i]], lwd=2, xlim=c(minimo-1, massimo+1),ylim=c(0,opt), col="blue", add=TRUE)
+            if(is.null(plot.name)){
+                title(main = paste("Variable X", i,sep=""))
+            }else{
+                title(main = plot.name[i])
+            }
         }
     }
 )
@@ -123,25 +132,37 @@ setMethod(
 
 ## ***************************************************************************************************
 
-
-CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n.marg), plot.marg = TRUE, plot.bar = TRUE, plot.legend = TRUE,
-                    args.legend = list(y = 110, cex = 0.8), model = list(normalCopula(0.5, dim=n.marg, dispstr="ex"), claytonCopula(10, dim=n.marg),
-                    gumbelCopula(10, dim=n.marg),frankCopula(10, dim=n.marg)),...){
+CoImp <- function(X, n.marg = ncol(X), x.up = NULL, x.lo = NULL, q.up = NULL, q.lo= NULL, type.data = "continuous",
+ smoothing = rep(0.5,n.marg), plot = TRUE, model = list(normalCopula(0.5, dim=n.marg, dispstr="ex"),
+ claytonCopula(10, dim=n.marg), gumbelCopula(10, dim=n.marg), frankCopula(10, dim=n.marg)),...){
     #
     if(!is.matrix(X))
-        stop("Data entry should be a matrix")
+        stop("Tha data should be a matrix")
     if(n.marg<=1)
-        stop("Data matrix should contain at least two variables")
+        stop("The data matrix should contain at least two variables")
     if(nrow(X)<1)
-        stop("Data matrix should contain at least one observation")
+        stop("The data matrix should contain at least one observation")
     if(any(is.na(X))==FALSE)
-        stop("Data matrix should contain at least one missing value")
+        stop("The data matrix should contain at least one missing value")
+    if(sum(complete.cases(X))==0)
+        stop("The data matrix should contain at least one complete record")
     if(type.data != "discrete" & type.data != "continuous")
-        stop("Data must be continuous or discrete")
-    if(plot.bar==FALSE & plot.legend==TRUE)
-        stop("Cannot produce legend plot without the plot!")
+        stop("The data must be either continuous or discrete")
     if(type.data == "discrete")
-        warning("The variables were treated as continuoue and then round off them.")
+        warning("The variables are treated as continuous and rounded off.")
+    #
+    if(!is.null(x.up) & !is.null(q.up))
+        stop("Specify either x.up xor q.up")
+    if(!is.null(x.lo) & !is.null(q.lo))
+        stop("Specify either x.lo xor q.lo")
+    if(!is.null(q.up)&(q.up<=0 || q.up>=1))
+        stop("q.up must lie in (0, 1)")
+    if(!is.null(q.lo)&(q.lo<=0 || q.lo>=1))
+        stop("q.lo must lie in (0, 1)")
+    if(is.null(x.up) & is.null(q.up))
+        q.up <- rep(0.999,n.marg)
+    if(is.null(x.lo) & is.null(q.lo))
+        q.lo <- rep(0.001,n.marg)
     #
     fit0_nn <- list()
     for(i in 1:n.marg){
@@ -159,46 +180,24 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
         f.x[[j]] <- function(x){predict(fit0[[j]], newdata=x)/as.numeric(knorm[[1]])}
         F.x[[j]] <- function(x){ifelse(!is.na(x),integrate(f.x[[j]], lower=0, upper=x)$value,return(NA))}
     }
-    #
-    if(plot.marg == TRUE){
-        if(n.marg<=6){
-            par(mfrow=c(2,ceiling(n.marg/2)))
-        }else{
-            par(mfrow=c(3,ceiling(n.marg/3)))
-        }
-        par(mai=c(0.5,0.5,0.3,0.5))
-        for(i in 1:n.marg){
-            j       <- i
-            minimo  <- min(X[complete.cases(X[,i]),i])-1
-            massimo <- max(X[complete.cases(X[,i]),i])+1
-            opt1     <- f.x[[i]](optimize(f.x[[i]], c(minimo,massimo), maximum=TRUE)$maximum)
-            his <- hist(X[,i], plot=FALSE)
-            opt2 <- max(his$density)
-            opt <- max(opt1,opt2)
-            plot(his, freq=FALSE, xlim=c(minimo, massimo), ylim=c(0,opt), main="", xlab="", ylab="")
-            plot(f.x[[i]], lwd=2, xlim=c(minimo, massimo), ylim=c(0,opt), col="blue", add=TRUE)
-            title(main = paste("Variable X", i,sep=""))
-        }
-    }
-    #
-    ind.miss <- which(is.na(X), arr.ind = TRUE)
+    ind.miss     <- which(is.na(X), arr.ind = TRUE)
     if(nrow(ind.miss)>1){ind.miss <-  ind.miss[order(ind.miss[,1]),]}
-    ind.cols.na <- split(ind.miss[,2],ind.miss[,1])
-    num.rows.na <- length(ind.cols.na)
+    ind.cols.na  <- split(ind.miss[,2],ind.miss[,1])
+    num.rows.na  <- length(ind.cols.na)
     #
-    n.mod <- length(model)
-    startco <- c(.5,rep(10,(n.mod-1)))
-    loglik <- double(length=n.mod)
-    metodo.fin <- double(length=n.mod)
+    n.mod        <- length(model)
+    startco      <- c(.5,rep(10,(n.mod-1)))
+    loglik       <- double(length=n.mod)
+    metodo.fin   <- double(length=n.mod)
     metodo.optim.fin <- double(length=n.mod)
     for(i in 1:n.mod){
-        metodo <- "ml"
+        metodo   <- "ml"
         metodo.c <- "BFGS"
-        udat.na <- fit.margin(dataset=t(X), param=list(dimc=n.marg))
-        udat <- udat.na[complete.cases(udat.na),]
-        fitc <- try(fitCopula(data=udat, copula=model[[i]], start=startco[i], method=metodo, optim.method=metodo.c), silent=TRUE)
+        udat.na  <- fit.margin(dataset=t(X), param=list(dimc=n.marg))
+        udat     <- udat.na[complete.cases(udat.na),]
+        fitc     <- try(fitCopula(data=udat, copula=model[[i]], start=startco[i], method=metodo, optim.method=metodo.c), silent=TRUE)
         if(inherits(fitc, "try-error")==TRUE){
-            metodo <- c("mpl","itau","irho")
+            metodo <- c("mpl", "itau", "irho")
             repeat{
                 if(length(metodo)==0 || inherits(fitc, "try-error")==FALSE)
                     break
@@ -223,10 +222,10 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
                 metodo.c <- setdiff(metodo.c, metodo.c[[1]])
             }
         }
-        if (inherits(fitc, "try-error") || is.nan(suppressWarnings(loglikCopula(param=fitc@estimate, x=udat, copula=model[[i]])))) {
+        if (inherits(fitc, "try-error") || is.nan(suppressWarnings(loglikCopula(param=fitc@estimate, u=udat, copula=model[[i]])))) {
             loglik[i] <- -10000
         }else{
-            loglik[i] <- suppressWarnings(loglikCopula(param=fitc@estimate, x=udat, copula=model[[i]]))
+            loglik[i] <- suppressWarnings(loglikCopula(param=fitc@estimate, u=udat, copula=model[[i]]))
         }
         if(length(metodo)==0){
             metodo.fin[i] <- 0
@@ -258,6 +257,14 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
     }
     #
     dati.fin <- X
+    x.min  <- apply(X,2,min, na.rm=TRUE)
+    x.max  <- apply(X,2,max, na.rm=TRUE)
+    if(is.null(x.up)){
+        x.up <- apply(X,2,quantile,p=q.up, na.rm=TRUE)
+    }
+    if(is.null(x.lo)){
+        x.lo <- apply(X,2,quantile,p=q.lo, na.rm=TRUE)
+    }
     for(i in 1:num.rows.na){
         cat("\r Number of imputed rows: ", i, "\n");
         cols.na <- ind.cols.na[[i]]
@@ -267,7 +274,7 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
         lcn <- length(cols.na)
         #
         meanX   <- colMeans(X,na.rm=TRUE)
-        medianX <- apply(X=X,FUN=median,MARGIN=2,na.rm=TRUE)
+        medianX <- apply(X=X, FUN=median, MARGIN=2, na.rm=TRUE)
         #
         zz <- y
         fcond <- function(x){
@@ -279,11 +286,12 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
         if(lcn==n.marg)
             stop("It cannot impute a full NA record")
         #
+        a  <- x.min[cols.na]
+        b  <- x.max[cols.na]
+        aa <- x.lo[cols.na]
+        bb <- x.up[cols.na]
+
         if(lcn==1){
-            a <- min(X[-which(is.na(X[,cols.na])),cols.na])
-            b <- max(X[-which(is.na(X[,cols.na])),cols.na])
-            aa <- quantile(X[-which(is.na(X[,cols.na])),cols.na],p=0.20)
-            bb <- quantile(X[-which(is.na(X[,cols.na])),cols.na],p=0.80)
             maxf <- suppressWarnings(optimize(fcond2, interval=c(a, b), maximum=TRUE)$max)
             if(maxf>=aa & maxf<=bb){
                 umissM <- hitormiss(FUN=fcond,p=lcn,h=fcond(maxf),a=aa,b=bb)
@@ -291,10 +299,6 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
                 umissM <- hitormiss(FUN=fcond,p=lcn,h=fcond(maxf),a=a,b=b)
             }
         }else{
-            a <- apply(X[-which(is.na(X[,cols.na])==TRUE,arr.ind=TRUE),cols.na],2,min)
-            b <- apply(X[-which(is.na(X[,cols.na])==TRUE,arr.ind=TRUE),cols.na],2,max)
-            aa <- apply(X[-which(is.na(X[,cols.na])==TRUE,arr.ind=TRUE),cols.na],2,quantile,p=0.20)
-            bb <- apply(X[-which(is.na(X[,cols.na])==TRUE,arr.ind=TRUE),cols.na],2,quantile,p=0.80)
             maxf <- suppressWarnings(try(optim(par=meanX[cols.na], fn=fcond, lower=a, upper=b, control = list(fnscale=-1))$par,silent=TRUE))
             if(inherits(maxf, "try-error")==TRUE){
                 maxf <- suppressWarnings(optim(par=medianX[cols.na], fn=fcond, lower=a, upper=b, control = list(fnscale=-1))$par)
@@ -346,10 +350,10 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
                 metodo.c <- setdiff(metodo.c, metodo.c[[1]])
             }
         }
-        if (inherits(fitc.imp, "try-error") || is.nan(suppressWarnings(loglikCopula(param=fitc.imp@estimate, x=udat.imp, copula=model[[i]])))) {
+        if (inherits(fitc.imp, "try-error") || is.nan(suppressWarnings(loglikCopula(param=fitc.imp@estimate, u=udat.imp, copula=model[[i]])))) {
             loglik.imp[i] <- -10000
         }else{
-            loglik.imp[i] <- suppressWarnings(loglikCopula(param=fitc.imp@estimate, x=udat.imp, copula=model[[i]]))
+            loglik.imp[i] <- suppressWarnings(loglikCopula(param=fitc.imp@estimate, u=udat.imp, copula=model[[i]]))
         }
         if(length(metodo)==0){
             metodo.fin.imp[i] <- 0
@@ -385,12 +389,6 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
     rownames(perc.data) <- c("Data","Missing")
     ifelse(is.null(colnames(X)), colnames(perc.data) <- paste("X",c(1:n.marg),sep=""), colnames(perc.data) <- colnames(X))
     #
-    if(plot.bar==TRUE){
-        dev.new();
-        par(mfrow=c(1,1));
-        plot.miss <- bar.plot(X, tab = perc.data, legend.plot = plot.legend, args.legend = args.legend)
-    }
-    #
     if(is.null(colnames(X))) colnames(X) <- paste("X",c(1:n.marg),sep="")
     ifelse(is.null(colnames(X)), colnames(dati.fin) <- paste("X",c(1:n.marg),sep=""), colnames(dati.fin) <- colnames(X))
     #
@@ -407,5 +405,6 @@ CoImp <- function(X, n.marg = 2, type.data = "continuous", smoothing = rep(0.5,n
     out@Imputed.data.matrix   <- dati.fin;
     out@Estimated.Model.Imp   <- mod.post;
     out@Estimation.Method.Imp <- metodo.fin.base.imp;
+    if(plot==TRUE) plot(out)
     return(out);
 }
